package track21;

import MainGUI;

import waba.io.*;
import java.lang.String;
import waba.sys.*;
import waba.util.*;

//import extra.io.*;
//import extra.io.builtin.*;

public class CatalogIO
{
  String dbPrefix="Trk2_";
//  String dbExtension=".CRTR.DATA";
  String dbExtension;

  String dbMaster="Master";
  String dbTimer="Timer_";
  String productID="";
  int numDB=0;
  String dbList[]=new String[25];

  public CatalogIO()
  {
    configProduct();
  }

  public void readDBlist()
  {
    Catalog c=openFileForRead(dbMaster);

    if(c.isOpen()==false)   //if masterDB doesn't exist, create it
    {
      c=openFileForCreate(dbMaster);
      c.close();
      return;
    }

    DataStream ds=new DataStream(c);
    numDB=c.getRecordCount();
    for (int i = 0; i < numDB; i++)
    {
      c.setRecordPos(i);
      dbList[i]=ds.readString();
    }
    c.close();

//    Vm.debug("ReadDBlist:");
//    debugDBlist();
  }


  public int getDBcount()
  {
    return numDB;
  }

  public String getDBname(int idx)
  {
    return dbList[idx];
  }

  public void addDBtoList(String dbName)
  {
    dbList[numDB]=dbName;
    numDB++;

    Catalog c=openFileForCreate(dbName);
    c.close();

    writeDBlist();
  }


  public void removeDBfromList(String dbName)
  {
    int delIdx=-1;

    for (int i = 0; i < numDB; i++)
    {
      if(dbName.compareTo(dbList[i])==0)
        delIdx=i;
    }

    if(delIdx!=-1)
    {
      for (int i = delIdx; i < numDB-1; i++)
      {
        dbList[i]=dbList[i+1];
      }

      numDB--;
      writeDBlist();
      deleteFile(dbName);
    }
  }

  public void writeDBtimeStamp(DateTimeStamp timestamp, String curDB)
  {
//    Vm.debug("Write DBlist");
//    debugDBlist();

    removeDBtimeStamp(curDB);

    Catalog c=openFileForCreate(getDBtimerFileName(curDB));

    ResizeStream rs=new ResizeStream(c,128);
    DataStream ds=new DataStream(rs);

    rs.startRecord();
    ds.writeInt(timestamp.getDateStampInt());
    ds.writeInt(timestamp.getTimeStampInt());
    rs.endRecord();

    c.close();
  }

  String getDBtimerFileName(String curDB)
  {
    String tStampFile=dbTimer+curDB;
    return tStampFile;
  }

  public void removeDBtimeStamp(String curDB)
  {
    deleteFile(getDBtimerFileName(curDB));
  }

  public DateTimeStamp readDBtimeStamp(String curDB)
  {
    Catalog c=openFileForRead(getDBtimerFileName(curDB));

    if(c.isOpen()==false)   //if masterDB doesn't exist, create it
      return null;    //what happens on PDA

    if(c.getRecordCount()==0)
      return null;    //what happens in applet (due to java security policy, can't delete file)

    DataStream ds=new DataStream(c);
    c.setRecordPos(0);
    int tmpDateInt=ds.readInt();
    int tmpTimeInt=ds.readInt();
    c.close();

    return new DateTimeStamp(tmpDateInt,tmpTimeInt);
  }


  public void writeDBlist()
  {
//    Vm.debug("Write DBlist");
//    debugDBlist();

    deleteFile(dbMaster);

    Catalog c=openFileForCreate(dbMaster);

    ResizeStream rs=new ResizeStream(c,128);
    DataStream ds=new DataStream(rs);

    for (int i = 0; i < numDB; i++)
    {
      rs.startRecord();
      ds.writeString(dbList[i]);
      rs.endRecord();
    }
    c.close();
  }


  public void writeDB(String dbName, TrackData tdIn)
  {
//    Vm.debug("WriteDB="+dbName);
//    tdIn.debug();

    deleteFile(dbName);

    Catalog  c=openFileForCreate(dbName);

    ResizeStream rs=new ResizeStream(c,128);
    DataStream ds=new DataStream(rs);

    rs.startRecord();
    ds.writeInt(tdIn.getGraphOptions().getXmin());
    ds.writeInt(tdIn.getGraphOptions().getXmax());
    ds.writeFloat(tdIn.getGraphOptions().getYmin());
    ds.writeFloat(tdIn.getGraphOptions().getYmax());
    ds.writeBoolean(tdIn.getGraphOptions().getXminAutoScl());
    ds.writeBoolean(tdIn.getGraphOptions().getXmaxAutoScl());
    ds.writeBoolean(tdIn.getGraphOptions().getYminAutoScl());
    ds.writeBoolean(tdIn.getGraphOptions().getYmaxAutoScl());
    ds.writeInt(tdIn.getGraphOptions().getXticks());
    ds.writeInt(tdIn.getGraphOptions().getYticks());
    ds.writeInt(tdIn.getGraphOptions().getPrecision());
    ds.writeBoolean(tdIn.getGraphOptions().getXisRelative());
    ds.writeInt(tdIn.getGraphOptions().getGraphType());
    ds.writeString(tdIn.getGraphOptions().getXtitle());
    ds.writeString(tdIn.getGraphOptions().getYtitle());
    ds.writeString(tdIn.getGraphOptions().getMainTitle());
    rs.endRecord();

    for (int i = 0; i < tdIn.getNumPts(); i++)
    {
      rs.startRecord();
      ds.writeInt(tdIn.getDateInt(i));
      ds.writeFloat(tdIn.getYvalue(i));
      ds.writeString(tdIn.getComment(i));
      rs.endRecord();
    }
    c.close();
  }

  void deleteFile(String fName)
  {
    Catalog c=openFileForWrite(fName);
    if(c.isOpen())  //if DB exists, delete it
    {
      c.delete();
    }
    c.close();
  }

  Catalog openFileForRead(String fName)
  {
    Catalog c=new Catalog(dbPrefix+productID+fName+dbExtension,Catalog.READ_ONLY);
    return c;
  }

  Catalog openFileForWrite(String fName)
  {
    Catalog c=new Catalog(dbPrefix+productID+fName+dbExtension,Catalog.WRITE_ONLY);
    return c;
  }

  Catalog openFileForCreate(String fName)
  {
    Catalog c=new Catalog(dbPrefix+productID+fName+dbExtension,Catalog.CREATE);
    return c;
  }

  public void readDB(String dbName, TrackData tdIn)
  {
    tdIn.clearAllData();
    Catalog c=openFileForRead(dbName);

//    if(c.isOpen()==false)
//      Vm.debug("DB="+dbName+" is not present!");

    DataStream ds=new DataStream(c);

    int tmpDate;
    float tmpYVal;
    int tmpXVal;
    String tmpComment;

    c.setRecordPos(0);    //record 0 is graph options
    tdIn.getGraphOptions().setXmin(ds.readInt());
    tdIn.getGraphOptions().setXmax(ds.readInt());
    tdIn.getGraphOptions().setYmin(ds.readFloat());
    tdIn.getGraphOptions().setYmax(ds.readFloat());
    tdIn.getGraphOptions().setXminAutoScl(ds.readBoolean());
    tdIn.getGraphOptions().setXmaxAutoScl(ds.readBoolean());
    tdIn.getGraphOptions().setYminAutoScl(ds.readBoolean());
    tdIn.getGraphOptions().setYmaxAutoScl(ds.readBoolean());
    tdIn.getGraphOptions().setXticks(ds.readInt());
    tdIn.getGraphOptions().setYticks(ds.readInt());
    tdIn.getGraphOptions().setPrecision(ds.readInt());
    tdIn.getGraphOptions().setXisRelative(ds.readBoolean());
    tdIn.getGraphOptions().setGraphType(ds.readInt());
    tdIn.getGraphOptions().setXtitle(ds.readString());
    tdIn.getGraphOptions().setYtitle(ds.readString());
    tdIn.getGraphOptions().setMainTitle(ds.readString());

    int tmpNumPts=c.getRecordCount()-1;
    for (int i = 0; i < tmpNumPts; i++)
    {
      c.setRecordPos(i+1);
      tmpXVal=ds.readInt();
      tmpYVal=ds.readFloat();
      tmpComment=ds.readString();
      tdIn.addData(tmpXVal,tmpYVal,tmpComment);
    }
    c.close();

//    Vm.debug("ReadDB="+dbName);
//    tdIn.debug();
  }

  void debugDBlist()
  {
//    Vm.debug("DBlist:");
    for (int i = 0; i < numDB; i++)
    {
      Vm.debug(i+": "+dbList[i]);
    }
  }


  void configProduct()
  {
    switch (MainGUI.currentProduct)
    {
      case MainGUI.PRODUCT_TIMEIT:
        productID="TI_";
        dbExtension=".TrkU.DATA";   //db extensions should match up with creator in make file
        break;
//      case MainGUI.PRODUCT_HTWT_BABY:
//        productID="HWB_";
//        dbExtension=".TrkB.DATA";
//        break;
//      case MainGUI.PRODUCT_HTWT_KID:
//        productID="HWK_";
//        dbExtension=".TrkK.DATA";
//        break;
      case MainGUI.PRODUCT_MONEY:
        productID="FI_";
        break;
      case MainGUI.PRODUCT_STUDENT:
        productID="ST_";
        break;
      case MainGUI.PRODUCT_HABIT:
        productID="HBT_";
        dbExtension=".Trk2.DATA";
        break;
      case MainGUI.PRODUCT_PLOTIT:
        productID="PI_";
        dbExtension=".TrkP.DATA";
        break;
      case MainGUI.PRODUCT_FULL:
        productID="FU_";
        dbExtension=".TrkF.DATA";
        break;
    }
  }


}
