package track21;

import waba.ui.*;
import waba.fx.*;
import waba.util.*;
import waba.sys.*;

//this class encapsulates the fitting alogorithm (least squares)
//and the associated controls
public class LineFit  extends Container {

	float a;
	float b;
	TrackData tdArr;
	boolean isDirty = false;
	Label label;  //shows the fitted parameters
	Button dateButton; //the project date button
	Label projLabel; //projected value label
	Check plotCheck; //check box to select plotting on the graph of the fitted line

	Calendar cal; //a popup calendar

	Window owner; //the mainwindow
	LineGraph graph; //the linegraph

	int lastDate; //this is the last date projected

	static final int NUMDECPLACES = 5; //numbers are all truncated to 5 dec. places

	public void setData(TrackData tmpTd)
  	{
    	tdArr=tmpTd;
    	isDirty = true;
  	}

  	//constructor
  	public LineFit(Window window, LineGraph gr)
  	{
  		owner = window;
  		graph = gr;
  		graph.setFit(this);

  		lastDate = 0;
  	}

  	//create the controls
  	public void onStart()
  	{
  	    //fill up the tab
  		setRect(0, 0, FILL, FILL);


  		//add a date button
  		//trackit uses US style dates, so this wont necessarily be the same format
  		Date tmpDate=new Date();
        dateButton=new Button(TrackData.dateIntToString(tmpDate.getDateInt()));
        add(dateButton);
        dateButton.setRect(5, BOTTOM - 5, PREFERRED, PREFERRED);

        lastDate = tmpDate.getJulianDay();

        //create a project value label
        projLabel = new Label("0.0");
        add(projLabel);
        projLabel.setRect(AFTER + 5, SAME, FILL, PREFERRED);

        //create a little title label
  		Label tmpLbl = new Label("Project value");
  		add(tmpLbl);
  		tmpLbl.setRect(5, BEFORE - 2, PREFERRED, PREFERRED);

  		//now create the plot controls
  		plotCheck = new Check("Plot fitted line");
  		plotCheck.setChecked(false);
  		add(plotCheck);
  		plotCheck.setRect(5, BEFORE - 5, PREFERRED, PREFERRED);


        //create the label to display parameters
  		label = new Label("");
  		add(label);
  		label.setRect(5, 5, FILL, FIT);

  	}

  	//handle events
  	public void onEvent(Event event)
    {
      switch (event.type)
      {
    	//if somethings pressed ...
    	case ControlEvent.PRESSED:
    	    //and its our date button then ...
            if (event.target == dateButton)
            {
              //popup the calendar
              if(cal==null)
                cal=new Calendar();
              owner.popupModal(cal);
            }
            else if (event.target == plotCheck)
            {
            	graph.plotFit(plotCheck.getChecked());
            }
            break;
        case ControlEvent.WINDOW_CLOSED:
            //if a window closed then its gonna be the calendar
            if(event.target==cal)
            {
              if (cal.canceled == false)
              {
                Date tmpDate = new Date();
                tmpDate = null;
                tmpDate = cal.getSelectedDate();
                //check for null date in case clear was pressed
                if (tmpDate != null)
                {
              	  dateButton.setText(TrackData.dateIntToString(tmpDate.getDateInt()));

              	  if (tdArr != null && tdArr.numPts > 1)
              	  {
              		if (isDirty)
              			doFit();

                    lastDate = tmpDate.getJulianDay();
              		projLabel.setText(Convert.toString(projectY(lastDate), NUMDECPLACES));
              	  }
              	 }
               }
             }
            break;
      }
    }

  	public void onPaint(Graphics g)
  	{

  		FontMetrics fm = getFontMetrics(g.getFont());
  		int textHeight = fm.getHeight();
  		int textOffset = 5;
  		String str;
  		if (tdArr.numPts > 1)
  		{

  		    //only do the fit if something's changed.
  			if (isDirty)
  			{
  				doFit();
  			}

  			str = "Fitted Line Y = A + BX: |";
  			str += "A = " + Convert.toString(a, NUMDECPLACES) + "|";
  			str += "B = " + Convert.toString(b, NUMDECPLACES) + "|";

  		    //making a guess the user is not going to project that far in the past!
  		    if (lastDate != 0)
  		       projLabel.setText(Convert.toString(projectY(lastDate), NUMDECPLACES));

  		}
  		else
  		{
  			str = "Two or more points are | required for a fit";

  			//reset the last projected date
  			lastDate = 0;
  			projLabel.setText("0.0");
  		}
  		label.setText(str);

  		super.onPaint(g);
  	}

  	public void doFit()
  	{
  		//calculate the line y = a + bx

  		//use x values relative to the first day of data entered.
  		//int xmin = ;
  		int xmin = tdArr.getDateIntMin();
  		xmin = new Date(xmin).getJulianDay();

  		float sumY  = 0; //sum of Y values
  		float sumX2 = 0; //sum of X^2 values
  		float sumX  = 0; //sum of X values
  		float sumXY = 0; //sum of X*Y values
  		int x;
  		float y;
  		for (int i = 0; i < tdArr.numPts; i++)
      	{

      	   //x = new Date((int)tdArr.getDateInt(i)).getJulianDay() - xmin;
      	   x = (int)tdArr.getDateInt(i);
      	   x = new Date(x).getJulianDay();
      	   x -= xmin;
      	   y = tdArr.getYvalue(i);
	       sumY += y;
	       sumX += x;
	       sumX2 += x * x;
	       sumXY += x * y;
      	}

      	//calcuate the denominator in the parameter calculation
        float denom = (tdArr.numPts * sumX2) - (sumX * sumX);

      	a = ((sumY * sumX2) - (sumX * sumXY)) /denom;
      	b = ((tdArr.numPts * sumXY) - (sumX * sumY)) /denom;

      	isDirty = false;
  	}

  	//calculate y given x in Julian Days from the fitted line.
  	public float projectY(float x)
  	{
  		float xmin = new Date(tdArr.getDateIntMin()).getJulianDay();
  		return a + b * (x - xmin);
  	}



}
