package track21;

import waba.ui.*;
import waba.fx.*;
import waba.util.*;
import waba.sys.*;

public class LineGraph extends Control
{
  Graphics graph;
  int lastX, lastY;
  int startX, startY;
  TrackData tdArr[];
  GraphOptions go;
  double xmin, xmax, xrange;
  float ymin, ymax,yrange;
  int xminJulianDay=0;

  boolean doFit; //true if we should show the fitted line
  LineFit fit; //fitted line object

  public LineGraph()
  {
  	doFit = false;
  }

  public void setData(TrackData tmpTd[])
  {
    tdArr=tmpTd;
  }

  public void onEvent(Event event)
    {
      if (event.type == PenEvent.PEN_DOWN)
        {
          PenEvent penEvent = (PenEvent)event;
          lastX = penEvent.x;
          lastY = penEvent.y;
          startX=lastX;
          startY=lastY;
        }
      else if (event.type == PenEvent.PEN_DRAG)
        {
          graph.setForeColor(0,0,0);
          PenEvent penEvent = (PenEvent)event;
          graph.drawLine(lastX, lastY, penEvent.x, penEvent.y);
          lastX = penEvent.x;
          lastY = penEvent.y;
        }
    }


   public void onPaint(Graphics g)
   {
      boolean plotSucessful=false;

      if (graph == null) graph = createGraphics();
      for (int i = 0; i < tdArr.length; i++)
      {
        boolean initGraph=(i==0);
//        Vm.debug("Plotting graph["+i+"]");
        if(paintGraph(tdArr[i], initGraph, graph))
          plotSucessful=true;
      }

      if (!plotSucessful)
      {
        g.drawText("A graph won't display graph until",5,60);
        g.drawText("you have at least 2 data points",10,70);
      }
   }


   boolean paintGraph(TrackData td, boolean initGraph, Graphics g)
   {
//      Vm.debug("paintGraph");
//      g.setBackColor(0,0,0);
//      g.fillHatchedRect(0,0,this.width,this.height,true,true);

      g.setForeColor(0,0,0);
      g.setBackColor(255,255,255);


      if(td.getNumPts()<2)
      {
        return false;
      }

      go=td.getGraphOptions();
      go.updateGraphMinMax();

//      go.setXmin(td.getDateIntMin());
//      go.setXmax(td.getDateIntMax());
//      go.setYmin(td.getYmin());
//      go.setYmax(td.getYmax());

      int x0=20, y0=11, x1=154, y1=115, yScrRange=y1-y0, xScrRange=x1-x0;
      int xticknum=0, yticknum=0;

      //fitted Y values for xmin and xmax
      float fitVal1 = 0;
      float fitVal2 = 0;

      if(initGraph)
      {
//        Vm.debug("Init Graph");
        g.drawRect(0,0,this.width,this.height);

        g.drawRect(x0,y0,x1-x0,y1-y0);

        xmin=go.getXmin();
        xminJulianDay=new Date((int)xmin).getJulianDay();
        xmax=go.getXmax();
        xrange=new Date((int)xmax).getJulianDay()-xminJulianDay;

        ymin=go.getYmin();
        ymax=go.getYmax();

        //calculate new ymax and ymin if we're plotting the fitted line
        if (doFit)
        {
        	//update the fitted parameters
        	fit.doFit();

        	fitVal1 = fit.projectY(new Date(td.getDateIntMin()).getJulianDay());
        	fitVal2 = fit.projectY(new Date(td.getDateIntMax()).getJulianDay());

        	//slope is positive
        	if (fitVal1 < fitVal2)
        	{
        	  if (ymin > fitVal1) ymin = fitVal1;
        	  if (ymax < fitVal2) ymax = fitVal2;
        	}
        	else //slope is negative
        	{
        	  if (ymin > fitVal2) ymin = fitVal2;
        	  if (ymax < fitVal1) ymax = fitVal1;
        	}
        }

        if(ymax==ymin)   //hack, if min=max, avoid div by zero error
          ymax=ymin+1;

        yrange=ymax-ymin;

        xticknum=go.getXticks();
        yticknum=go.getYticks();
      }

//      Vm.debug("Data Xmin="+td.getDateIntMin()+"Xmax="+td.getDateIntMax()+", Graph Xmin="+xmin+" Graph Xmax="+xmax);
//      Vm.debug("Data Ymin="+td.getYmin()+"Ymax="+td.getYmax()+", Graph Ymin="+ymin+" Graph Ymax="+ymax);

      double oldx=0;
      float oldy=0;

      int offsetDays=0;

      if(td.getGraphOptions().getXisRelative() && !initGraph)
        offsetDays=xminJulianDay-new Date(td.getDateIntMin()).getJulianDay();

      for (int i = 0; i < td.getNumPts(); i++)
      {
        float y=1f-(td.getYvalue(i)-ymin)/yrange;
        y=y*yScrRange+y0;

        double x=(new Date(td.getDateInt(i)).getJulianDay()+offsetDays-xminJulianDay)/(double)xrange;
//        Vm.debug("OffsetDays="+offsetDays+" X="+x);
        x=x*xScrRange+x0;

//        Vm.debug("X="+x+" Y="+y);
        g.drawCircle((int)x,(int)y,1);

        if((go.getGraphType() & GraphOptions.GTYPE_DOTS)==GraphOptions.GTYPE_DOTS)
          g.drawCircle((int)x,(int)y,2);

        if(i>0)
          if((go.getGraphType() & GraphOptions.GTYPE_LINE)==GraphOptions.GTYPE_LINE)
            g.drawLine((int)oldx,(int)oldy,(int)x,(int)y);

        if((go.getGraphType() & GraphOptions.GTYPE_BAR)==GraphOptions.GTYPE_BAR)
            g.drawLine((int)x,y1,(int)x,(int)y);

        oldx=x;
        oldy=y;
      }

      //now draw the  fitted line (its always dotted)
      if (doFit)
      {
      	float fy1 = 1f - (fitVal1 - ymin)/yrange;
      	fy1 = fy1 * yScrRange + y0;

      	double fx1 = offsetDays / (double)xrange;
      	fx1 = fx1 * xScrRange + x0;

      	float fy2 = 1f - (fitVal2 - ymin)/yrange;
      	fy2 = fy2 * yScrRange + y0;

      	double fx2 = (new Date(td.getDateIntMax()).getJulianDay()+offsetDays-xminJulianDay)/(double)xrange;
      	fx2 = fx2 * xScrRange + x0;


      	g.drawDots((int)fx1, (int)fy1, (int)fx2, (int)fy2);
      }

//      Vm.debug("Graph:");
//      td.debug();


    if (initGraph)
    {
      if (xticknum>0)
      {
        for (int i = 0; i < (xticknum+1); i++)
        {
          x=(xScrRange/(xticknum+1))*i+x0;
          g.drawLine(x,y0,x,y1);
        }
      }

      if (yticknum>0)
      {
        for (int i = 0; i < (yticknum+1); i++)
        {
          y=(yScrRange/(yticknum+1))*i+y0;
          g.drawLine(x0,y,x1,y);
        }
      }

      g.drawText(td.dateIntToString((int)xmin),2,y1);
      g.drawText(td.dateIntToString((int)xmax),x1-45,y1);

//      g.drawText(new String().valueOf(ymax),2,y0-4);
//      g.drawText(new String().valueOf(ymin),2,y1-8);
      g.drawText(TrackData.getYvalueStr(ymax,go.getPrecision()),2,y0-4);
      g.drawText(TrackData.getYvalueStr(ymin,go.getPrecision()),2,y1-8);

      g.drawText(td.getGraphOptions().getXtitle(),60,y1);
      g.drawText(td.getGraphOptions().getMainTitle(),30,1);
      for (int i = 0; i < td.getGraphOptions().getYtitle().length(); i++)
      {
        g.drawText(""+td.getGraphOptions().getYtitle().charAt(i),4,y0+20+i*9);
      }
    }

    return true;
   }

   //set whether to plot the fit
   public void plotFit(boolean showFit)
   {
   	doFit = showFit;
   }

   //set the fitter object
   public void setFit(LineFit lineFit)
   {
   	 fit = lineFit;
   }

}
