package track21;

import waba.sys.*;
import java.lang.String;

public class TrackData
{
  int maxDataPts=100;

  String comment[]=new String[maxDataPts];
//  double valX[]=new float[maxDataPts]; //should be float, but not enough bits in mantissa
  int valX[]=new int[maxDataPts];   //stored as YYYYMMDD
  float valY[]=new float[maxDataPts];
  int numPts=0;
  boolean xIsDate=true;

  float minY, maxY;
  int minX, maxX;

  GraphOptions gopts;

  public TrackData()
  {
    gopts=new GraphOptions(this);
  }


  public void setComment(String comStr,int idx)
  {
    comment[idx]=new String(comStr);
  }

  public String getComment(int idx)
  {
    return comment[idx];
  }


  public void setDateInt(int dateIn, int idx)
  {
    setXvalue(dateIn,idx);   //save Xval as YYYYMMDD
  }


  public int getDateInt(int idx)
  {
    return getXvalue(idx);
  }

  public void setYvalue(float valIn, int idx)
  {
    valY[idx]=valIn;
  }

  public void setXvalue(int valIn, int idx)
    {
      valX[idx]=valIn;
    }

  public float getYvalue(int idx)
  {
    return valY[idx];
  }

  public String getYvalueStr(int idx)
  {
    String tmpStr;
    if(getGraphOptions().getPrecision()==0)
      tmpStr=Convert.toString((int)getYvalue(idx));
    else
      tmpStr=Convert.toString(getYvalue(idx),getGraphOptions().getPrecision());
    return tmpStr;
  }

  public static String getYvalueStr(float floatIn, int precision)
  {
    String tmpStr;
    if(precision==0)
      tmpStr=Convert.toString((int)floatIn);
    else
      tmpStr=Convert.toString(floatIn,precision);
    return tmpStr;
  }

  public int getXvalue(int idx)
  {
    return valX[idx];
  }

  public int getNumPts()
  {
    return numPts;
  }

  public String buildTableStr(int idx)
  {
    String tmpStr=new String(dateIntToString(getDateInt(idx))+"   "+getYvalueStr(idx)+"   "+getComment(idx));
//    String tmpStr=Convert.toString(getXvalue(idx))+"   "+Convert.toString(getYvalue(idx),gopts.getPrecision()) +"   "+getComment(idx);
    return tmpStr;
  }

  public void removeData(int idx)
  {
    for (int i = idx; i < numPts; i++)
    {
//      setXvalue(getXvalue(i+1),i);
//      setYvalue(getYvalue(i+1),i);
//      setComment(getComment(i+1),i);

      valX[i]=valX[i+1];
      valY[i]=valY[i+1];
      comment[i]=comment[i+1];
    }

    numPts--;
    setMinMax();
  }

  public void removeDataBeforeIdx(int idx)
  {
    for (int i = idx; i < numPts; i++)
    {
//      setXvalue(getXvalue(i+1),i);
//      setYvalue(getYvalue(i+1),i);
//      setComment(getComment(i+1),i);

      valX[i-idx]=valX[i];
      valY[i-idx]=valY[i];
      comment[i-idx]=comment[i];
    }

    numPts=numPts-idx;
    setMinMax();
  }

  public void addData(int tmpX, float tmpY, String tmpString)
  {
    numPts++;
    setXvalue(tmpX,numPts-1);
    setYvalue(tmpY,numPts-1);
    setComment(tmpString,numPts-1);

//    Vm.debug("Added "+getNumPts()+":"+tmpX+","+tmpY);

    setMinMax();

    if (getNumPts()>1)
    {
      if (getDateInt(getNumPts()-2) > getDateInt(getNumPts()-1))  //if new point is out of order, sort data
      {
        sortData();
      }
    }

    if(numPts==(maxDataPts-1))    //if max number of points, remove the oldest one
      removeData(0);
  }

  public void clearAllData()
  {
    numPts=0;
  }

  public void consolidateData()   //expects data to be sorted!
  {
    for (int i = 0; i < numPts-1; i++)
    {
      if(getXvalue(i)==getXvalue(i+1))  //if X values are same, consolidate points
      {
        setYvalue(getYvalue(i)+getYvalue(i+1),i);
        if(getComment(i).length()!=0)
          setComment(getComment(i)+" "+getComment(i+1),i);
        else
          setComment(getComment(i+1),i);

        removeData(i+1);  //remove redundant data point
      }
    }
  }


  /**
   * Bubble-sort, yeech!
   */
  public void sortData()
  {
    Vm.debug("Sorting Data");
    for (int i = 0; i < (numPts-1); i++)
    {
      for (int j = i+1; j < numPts; j++)
      {
//        Vm.debug("Sort i="+i+" j="+j+" numpts="+numPts);
        if(getXvalue(i)>getXvalue(j))
        {
          int tmpX=getXvalue(j);
          float tmpY=getYvalue(j);
          String tmpComment=getComment(j);

          setXvalue(getXvalue(i),j);
          setYvalue(getYvalue(i),j);
          setComment(getComment(i),j);

          setXvalue(tmpX,i);
          setYvalue(tmpY,i);
          setComment(tmpComment,i);
        }
      }
    }
  }


  public void debug()
  {
//    Vm.debug("Num pts="+numPts);
    for (int i = 0; i < numPts; i++)
    {
      Vm.debug("Datadump: "+buildTableStr(i));
    }
  }


  void setMinMax()
  {
    minX=maxX=valX[0];
    minY=maxY=valY[0];

    for (int i = 0; i < numPts; i++)
    {
      if(minX>valX[i])
        minX=valX[i];
      if(maxX<valX[i])
        maxX=valX[i];
      if(minY>valY[i])
        minY=valY[i];
      if(maxY<valY[i])
        maxY=valY[i];
    }
  }

  public int getMaxPrecision()    //scans all values to set precision appropriately
  {
    int necessPrec=0;

    for (int i = 0; i < numPts; i++)
    {
      if(GraphOptions.calcPrecision(valY[i])>necessPrec)
        necessPrec=GraphOptions.calcPrecision(valY[i]);
    }
     return necessPrec;
  }

  public int getDateIntMin()
  {
    return minX;
  }

  public int getDateIntMax()
  {
    return maxX;
  }

  public float getYmin()
  {
    return minY;
  }

  public float getYmax()
  {
    return maxY;
  }

  public float getXmin()
  {
    return minX;
  }

  public float getXmax()
  {
    return maxX;
  }

  public static String dateIntToString(int tmpDateInt)
  {
    String retStr;

    int yr=(int)(tmpDateInt/10000);
    tmpDateInt=tmpDateInt-(yr*10000);
    int mo=(int)(tmpDateInt/100);
    tmpDateInt=tmpDateInt-(mo*100);
    int da=tmpDateInt;

    if(GraphOptions.getDMYformat()== GraphOptions.DFORMAT_MM_DD_YYYY)
      return(leadingZero(mo)+"-"+leadingZero(da)+"-"+yr);
    else
    if(GraphOptions.getDMYformat()== GraphOptions.DFORMAT_DD_MM_YYYY)
      return(leadingZero(da)+"-"+leadingZero(mo)+"-"+yr);

    return "Unknown Date Format";
  }

  //return 2-digit string with leading zero if needed
  static String leadingZero(int numIn)
  {
    if(numIn<10)
      return("0"+numIn);
    else
      return new String().valueOf(numIn);
  }

  public GraphOptions getGraphOptions()
  {
    return gopts;
  }

}
